@@ -153,7 +153,10 @@ class SingletonParam(type):
         cls.instance = {}
 
     def __call__(cls, *args, **kw):
-        keyarg = args[0] if args else ""
+        if hasattr(cls, "param_id"):
+            keyarg = cls.param_id(*args, **kw)
+        else:
+            keyarg = args[0] if args else ""
         if keyarg not in cls.instance:
             create_obj = getattr(super(SingletonParam, cls), '__call__')
             cls.instance[keyarg] = create_obj(*args, **kw)
